// -*- C++ -*-
// $RCSfile: progressbar.h,v $
// $Revision: 1.27 $
// $Author: vrc $
// $Date: 2007-09-26 22:02:22 $

/* This software was produced by NIST, an agency of the U.S. government,
 * and by statute is not subject to copyright in the United States.
 * Recipients of this software assume all responsibilities associated
 * with its operation, modification and maintenance. However, to
 * facilitate maintenance we ask that before distributing modified
 * versions of this software, you first contact the authors at
 * oof_manager@ctcms.nist.gov. 
 */

#include <oofconfig.h>

#ifndef PROGRESSBAR_H
#define PROGRESSBAR_H

#include <Python.h>
#include <string>
#include <pthread.h>
#include <sys/types.h>
#include <unistd.h>
#include "common/lock.h"

class ThreadState;
class ThreadID;

enum ProgressBarStatus {INCOMPLETE, COMPLETE, FAILED};


// C++ interface for the Progress object.

// TODO: This should NOT be in progressbar.*.  The ProgressBar is a
// gui or text representation of a Progress object.  The word "bar"
// should not appear in this file.

class Progress {
private:
  // For locking when reading and writing to member variables of the
  // progress bar
  mutable SLock lockobj;

  // progress stores the fraction of closeness to the completion of
  // the running process/thread.
  // 0: nothing done.
  // 1: progress completed.
  float progress;

  // if set to not zero (by a click or CTRL-C) when queried the thread
  // should be forced to stop and join.
  bool stop;

  ProgressBarStatus success; 
  
  std::string bar_msg;		// for graphics mode (TODO: Why is it here?)
  
  ThreadState *tState;		// pointer to the current threadstate
  
  void remove_from_list();
  Progress(const Progress&);	// prohibited!
  Progress();			// only called by getProgress().
public:
  ~Progress();
  // thread state controls
  void reset();
  bool query_stop() const;
  void stop_it();
  void set_success();
  void set_failure();
  ProgressBarStatus get_success() const;

  ThreadState *get_threadstate() const { return tState; }
  ThreadID & get_thread_ID() const;

  // gives or sets completion of process fraction with normalized 
  // values ranging from 0 to 1.
  float get_progress() const;
  void set_progress(float var);

  // text messaging. Common to both text and graphics progress bars
  void set_message(const std::string &msg);
  const std::string get_message() const;
  void cleanup();
  void make_defunct();

  // Factory function that returns the Progress object for the current
  // thread, or creates one if necessary.
  friend Progress *getProgress();
};


bool operator==(Progress &p1, Progress &p2);

Progress * getProgress();

#endif // PROGRESSBAR_H


